<?php

namespace Picea\Ui\Form;

use Picea\Ui\Common\UiElement,
    Picea\Extension\Extension,
    Picea\Extension\ExtensionTrait;

class UiForm extends UiElement implements Extension {
    use ExtensionTrait;

    public string $defaultMethod = "get";

    public array $token = [ "ui.form", "ui.endform", "ui.form.get", "ui.form.post", "ui.form.patch", "ui.form.delete", "ui.form.put" ];

    public string $tag = "form";

    public array $attributes = [
       'class' => 'ui-form',
    ];

    public function parse(\Picea\Compiler\Context &$context, ?string $arguments, string $token) : string
    {
        switch($token) {
            case 'ui.endform':
                return "</form>";

            case "ui.form.get":
                $method = "get";
            break;

            case "ui.form.patch":
                $method = "patch";
            break;

            case "ui.form.delete":
                $method = "delete";
            break;

            case "ui.form.put":
                $method = "put";
            break;

            case "ui.form.post":
               $method = "post";
            break;
        }

        $method ??= $this->defaultMethod;
        
        return "<?php echo ( new \\" . static::class . "() )->buildHtml('$method', $arguments) ?>";
    }
    
    public function buildHtml(string $method = "get", string $name = "", string $action = "", array $attributes = []) : string
    {
        $method = strtolower($method);

        $this->option('tag-type', 'single');
        

        if ($attributes['class'] ?? false) {
            $attributes['class'] .= " {$this->attributes['class']}";
            unset($this->attributes['class']);
        }
        
        $this->attributes([ 'method' => $method, 'action' => $action ] + $attributes);
        
        if ( $method !== "get" ) {
            $token = md5( $name . microtime());
            $key = "picea-ui.form:{$name}";

            if (count($_SESSION[$key] ?? []) > 100) {
                array_shift($_SESSION[$key]);
            }

            $_SESSION[$key][] = $token;

            $this->append( ( new UiHidden() )->attributes([ 
                'name' => "picea-ui-form[$name]",
                'value' => $token,
            ]));
            
            $this->attributes([ 'enctype' => "multipart/form-data" ]);
        }

        return $this->render() . PHP_EOL;
    }
}