<?php

namespace Picea\Ui\Method;

use Psr\Http\Message\ServerRequestInterface,
    Psr\Http\Message\ResponseInterface;

class FormContext implements FormContextInterface
{
    public string $formName;
    
    public bool $formSent = false;

    public bool $formExecuted = false;

    public mixed $formExecutionStatus = null;

    public array $values = [];

    public array $files = [];

    public array $messages = [];

    public bool $skipCsrf = false;
    
    public ServerRequestInterface $request;
    
    public ? ResponseInterface $response = null;

    public function __construct(ServerRequestInterface $request, ? string $formName = null)
    {
        $this->request = $request;

        if ( $formName ) {
            $this->formName = $formName;
        }

        $this->values = $request->getParsedBody() ?: [];

        if ( ! $this->values ) {
            $content = utf8_encode((string) $request->getBody());

            if ( $content && ( $json = json_decode($content, true) ) ) {
                $this->values = $json;
            }
        }

        $this->files = $request->getUploadedFiles() ?: [];
    }
    
    public function valid() : bool
    {
        foreach($this->messages as $message) {
            if ( $message->isError() ) {
                return false;
            }
        }
        
        return true;
    }

    public function executed() : bool
    {
        return $this->formExecuted;
    }

    public function formSent() : bool
    {
        $valid = in_array($this->request->getMethod(), [ 'POST', 'PUT', 'PATCH', 'DELETE', ]);

        if ( ! $this->skipCsrf && ($this->formName ?? false) ) {
            $token = $this->get('picea-ui-form')[$this->formName] ?? false;

            if ( $token ) {
                if ($this->validateCsrfToken) {
                    $valid = in_array($token, $_SESSION["picea-ui.form:{$this->formName}"] ?? []);
                }
                else {
                    $valid = (bool) $token;
                }
            }
            else {
                $valid = false;
            }
        }

        return $this->formSent = $valid;
    }

    public function __set($key, $value)
    {
        return $this->set($key, $value);
    }

    public function __get($key)
    {
        return $this->get($key);
    }

    public function __isset($key)
    {
        return array_key_exists($key, $this->values);
    }

    public function __unset($key)
    {
        $this->delete($key);
    }   
    
    public function get(string $key, $default = null)
    {
        return $this->has($key) ? $this->values[$key] : $default;
    }

    public function set(string $key, $value)
    {
        return $this->values[$key] = $value;
    }

    public function delete(string $key) : void
    {
        unset($this->values[$key]);
    }

    public function has(string $key) : bool
    {
        return array_key_exists($key, $this->values);
    }
    
    public function pushMessage(FormMessage $message) : void
    {
        $this->messages[] = $message;
    }
}